@@ -1,4 +1,4 @@
-# Copyright 2022-2024 MetaOPT Team. All Rights Reserved.
+# Copyright 2022-2025 MetaOPT Team. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -56,19 +56,25 @@ Point(x=2.0, y=6.0, z=3.0, norm=7.0)
 True
 """
 
+# pylint: disable=too-many-arguments
+
 from __future__ import annotations
 
 import contextlib
 import dataclasses
+import functools
 import inspect
 import sys
 import types
 from dataclasses import *  # noqa: F401,F403,RUF100 # pylint: disable=wildcard-import,unused-wildcard-import
-from typing import Any, Callable, Iterable, TypeVar, overload
-from typing_extensions import Literal  # Python 3.8+
+from typing import TYPE_CHECKING, Any, Callable, Literal, Protocol, TypeVar, overload
 from typing_extensions import dataclass_transform  # Python 3.11+
 
 
+if TYPE_CHECKING:
+    from collections.abc import Iterable
+
+
 # Redefine `field`, `dataclasses`, and `make_dataclasses`.
 # The remaining APIs are re-exported from the original package.
 __all__ = [*dataclasses.__all__]
@@ -84,7 +90,7 @@ _TypeT = TypeVar('_TypeT', bound=type)
 
 
 @overload  # type: ignore[no-redef]
-def field(  # pylint: disable=too-many-arguments
+def field(
     *,
     default: _T,
     init: bool = True,
@@ -93,13 +99,13 @@ def field(  # pylint: disable=too-many-arguments
     compare: bool = True,
     metadata: dict[Any, Any] | None = None,
     kw_only: bool | Literal[dataclasses.MISSING] = dataclasses.MISSING,  # type: ignore[valid-type] # Python 3.10+
+    doc: str | None = None,  # Python 3.14+
     pytree_node: bool | None = None,
-) -> _T:
-    """Field factory for :func:`dataclass`."""
+) -> _T: ...
 
 
 @overload
-def field(  # pylint: disable=too-many-arguments
+def field(
     *,
     default_factory: Callable[[], _T],
     init: bool = True,
@@ -108,12 +114,13 @@ def field(  # pylint: disable=too-many-arguments
     compare: bool = True,
     metadata: dict[Any, Any] | None = None,
     kw_only: bool | Literal[dataclasses.MISSING] = dataclasses.MISSING,  # type: ignore[valid-type] # Python 3.10+
+    doc: str | None = None,  # Python 3.14+
     pytree_node: bool | None = None,
 ) -> _T: ...
 
 
 @overload
-def field(  # pylint: disable=too-many-arguments
+def field(
     *,
     init: bool = True,
     repr: bool = True,  # pylint: disable=redefined-builtin
@@ -121,11 +128,12 @@ def field(  # pylint: disable=too-many-arguments
     compare: bool = True,
     metadata: dict[Any, Any] | None = None,
     kw_only: bool | Literal[dataclasses.MISSING] = dataclasses.MISSING,  # type: ignore[valid-type] # Python 3.10+
+    doc: str | None = None,  # Python 3.14+
     pytree_node: bool | None = None,
 ) -> Any: ...
 
 
-def field(  # pylint: disable=function-redefined,too-many-arguments
+def field(  # noqa: D417 # pylint: disable=function-redefined
     *,
     default: Any = dataclasses.MISSING,
     default_factory: Any = dataclasses.MISSING,
@@ -135,6 +143,7 @@ def field(  # pylint: disable=function-redefined,too-many-arguments
     compare: bool = True,
     metadata: dict[Any, Any] | None = None,
     kw_only: bool | Literal[dataclasses.MISSING] = dataclasses.MISSING,  # type: ignore[valid-type] # Python 3.10+
+    doc: str | None = None,  # Python 3.14+
     pytree_node: bool | None = None,
 ) -> Any:
     """Field factory for :func:`dataclass`.
@@ -177,11 +186,16 @@ def field(  # pylint: disable=function-redefined,too-many-arguments
         'metadata': metadata,
     }
 
-    if sys.version_info >= (3, 10):
+    if sys.version_info >= (3, 10):  # pragma: >=3.10 cover
         kwargs['kw_only'] = kw_only
-    elif kw_only is not dataclasses.MISSING:
+    elif kw_only is not dataclasses.MISSING:  # pragma: <3.10 cover
         raise TypeError("field() got an unexpected keyword argument 'kw_only'")
 
+    if sys.version_info >= (3, 14):  # pragma: >=3.14 cover
+        kwargs['doc'] = doc
+    elif doc is not None:  # pragma: <3.14 cover
+        raise TypeError("field() got an unexpected keyword argument 'doc'")
+
     if not init and pytree_node:
         raise TypeError(
             '`pytree_node=True` is not allowed for non-init fields. '
@@ -192,7 +206,7 @@ def field(  # pylint: disable=function-redefined,too-many-arguments
 
 
 @overload  # type: ignore[no-redef]
-def dataclass(  # pylint: disable=too-many-arguments
+def dataclass(
     *,
     init: bool = True,
     repr: bool = True,  # pylint: disable=redefined-builtin
@@ -205,13 +219,13 @@ def dataclass(  # pylint: disable=too-many-arguments
     slots: bool = False,  # Python 3.10+
     weakref_slot: bool = False,  # Python 3.11+
     namespace: str,
-) -> Callable[[_TypeT], _TypeT]:
-    """Dataclass decorator with PyTree integration."""
+) -> Callable[[_TypeT], _TypeT]: ...
 
 
 @overload
-def dataclass(  # pylint: disable=too-many-arguments
+def dataclass(
     cls: _TypeT,
+    /,
     *,
     init: bool = True,
     repr: bool = True,  # pylint: disable=redefined-builtin
@@ -228,8 +242,9 @@ def dataclass(  # pylint: disable=too-many-arguments
 
 
 @dataclass_transform(field_specifiers=(field,))
-def dataclass(  # noqa: C901 # pylint: disable=function-redefined,too-many-arguments,too-many-locals,too-many-branches
+def dataclass(  # noqa: C901,D417 # pylint: disable=function-redefined,too-many-locals,too-many-branches
     cls: _TypeT | None = None,
+    /,
     *,
     init: bool = True,
     repr: bool = True,  # pylint: disable=redefined-builtin
@@ -265,20 +280,20 @@ def dataclass(  # noqa: C901 # pylint: disable=function-redefined,too-many-argum
         'frozen': frozen,
     }
 
-    if sys.version_info >= (3, 10):
+    if sys.version_info >= (3, 10):  # pragma: >=3.10 cover
         kwargs['match_args'] = match_args
         kwargs['kw_only'] = kw_only
         kwargs['slots'] = slots
-    elif match_args is not True:
+    elif match_args is not True:  # pragma: <3.10 cover
         raise TypeError("dataclass() got an unexpected keyword argument 'match_args'")
-    elif kw_only is not False:
+    elif kw_only is not False:  # pragma: <3.10 cover
         raise TypeError("dataclass() got an unexpected keyword argument 'kw_only'")
-    elif slots is not False:
+    elif slots is not False:  # pragma: <3.10 cover
         raise TypeError("dataclass() got an unexpected keyword argument 'slots'")
 
-    if sys.version_info >= (3, 11):
+    if sys.version_info >= (3, 11):  # pragma: >=3.11 cover
         kwargs['weakref_slot'] = weakref_slot
-    elif weakref_slot is not False:
+    elif weakref_slot is not False:  # pragma: <3.11 cover
         raise TypeError("dataclass() got an unexpected keyword argument 'weakref_slot'")
 
     if cls is None:
@@ -297,7 +312,7 @@ def dataclass(  # noqa: C901 # pylint: disable=function-redefined,too-many-argum
     if namespace is not GLOBAL_NAMESPACE and not isinstance(namespace, str):
         raise TypeError(f'The namespace must be a string, got {namespace!r}.')
     if namespace == '':
-        raise ValueError('The namespace cannot be an empty string.')
+        namespace = GLOBAL_NAMESPACE
 
     cls = dataclasses.dataclass(cls, **kwargs)  # type: ignore[assignment]
 
@@ -321,6 +336,7 @@ def dataclass(  # noqa: C901 # pylint: disable=function-redefined,too-many-argum
 
     def flatten_func(
         obj: _T,
+        /,
     ) -> tuple[
         tuple[_U, ...],
         tuple[tuple[str, Any], ...],
@@ -330,25 +346,46 @@ def dataclass(  # noqa: C901 # pylint: disable=function-redefined,too-many-argum
         metadata = tuple((name, getattr(obj, name)) for name in metadata_fields)
         return children, metadata, children_field_names
 
-    def unflatten_func(metadata: tuple[tuple[str, Any], ...], children: tuple[_U, ...]) -> _T:  # type: ignore[type-var]
+    # pylint: disable-next=line-too-long
+    def unflatten_func(metadata: tuple[tuple[str, Any], ...], children: tuple[_U, ...], /) -> _T:  # type: ignore[type-var]
         kwargs = dict(zip(children_field_names, children))
         kwargs.update(metadata)
         return cls(**kwargs)
 
-    from optree.accessor import DataclassEntry  # pylint: disable=import-outside-toplevel
+    from optree.accessors import DataclassEntry  # pylint: disable=import-outside-toplevel
     from optree.registry import register_pytree_node  # pylint: disable=import-outside-toplevel
 
     return register_pytree_node(  # type: ignore[return-value]
         cls,
-        flatten_func,  # type: ignore[arg-type]
+        flatten_func,
         unflatten_func,  # type: ignore[arg-type]
         path_entry_type=DataclassEntry,
         namespace=namespace,
     )
 
 
-# pylint: disable-next=function-redefined,too-many-arguments,too-many-locals,too-many-branches
-def make_dataclass(  # type: ignore[no-redef] # noqa: C901
+class _DataclassDecorator(Protocol[_TypeT]):  # pylint: disable=too-few-public-methods
+    def __call__(  # pylint: disable=arguments-differ
+        self,
+        cls: _TypeT,
+        /,
+        *,
+        init: bool = True,
+        repr: bool = True,  # pylint: disable=redefined-builtin
+        eq: bool = True,
+        order: bool = False,
+        unsafe_hash: bool = False,
+        frozen: bool = False,
+        match_args: bool = True,
+        kw_only: bool = False,
+        slots: bool = False,
+        weakref_slot: bool = False,
+    ) -> _TypeT:
+        raise NotImplementedError
+
+
+# pylint: disable-next=function-redefined,too-many-locals,too-many-branches
+def make_dataclass(  # type: ignore[no-redef] # noqa: C901,D417
     cls_name: str,
     # pylint: disable-next=redefined-outer-name
     fields: Iterable[str | tuple[str, Any] | tuple[str, Any, Any]],
@@ -366,8 +403,9 @@ def make_dataclass(  # type: ignore[no-redef] # noqa: C901
     slots: bool = False,  # Python 3.10+
     weakref_slot: bool = False,  # Python 3.11+
     module: str | None = None,  # Python 3.12+
+    decorator: _DataclassDecorator[_TypeT] = dataclasses.dataclass,  # type: ignore[assignment] # Python 3.14+
     namespace: str,  # the PyTree registration namespace
-) -> type:
+) -> _TypeT:
     """Make a new dynamically created dataclass with PyTree integration.
 
     The dataclass name will be ``cls_name``. ``fields`` is an iterable of either (name), (name, type),
@@ -405,7 +443,7 @@ def make_dataclass(  # type: ignore[no-redef] # noqa: C901
     if namespace is not GLOBAL_NAMESPACE and not isinstance(namespace, str):
         raise TypeError(f'The namespace must be a string, got {namespace!r}.')
     if namespace == '':
-        raise ValueError('The namespace cannot be an empty string.')
+        namespace = GLOBAL_NAMESPACE
 
     dataclass_kwargs = {
         'init': init,
@@ -420,23 +458,23 @@ def make_dataclass(  # type: ignore[no-redef] # noqa: C901
         'namespace': ns,
     }
 
-    if sys.version_info >= (3, 10):
+    if sys.version_info >= (3, 10):  # pragma: >=3.10 cover
         dataclass_kwargs['match_args'] = match_args
         dataclass_kwargs['kw_only'] = kw_only
         dataclass_kwargs['slots'] = slots
-    elif match_args is not True:
+    elif match_args is not True:  # pragma: <3.10 cover
         raise TypeError("make_dataclass() got an unexpected keyword argument 'match_args'")
-    elif kw_only is not False:
+    elif kw_only is not False:  # pragma: <3.10 cover
         raise TypeError("make_dataclass() got an unexpected keyword argument 'kw_only'")
-    elif slots is not False:
+    elif slots is not False:  # pragma: <3.10 cover
         raise TypeError("make_dataclass() got an unexpected keyword argument 'slots'")
 
-    if sys.version_info >= (3, 11):
+    if sys.version_info >= (3, 11):  # pragma: >=3.11 cover
         dataclass_kwargs['weakref_slot'] = weakref_slot
-    elif weakref_slot is not False:
+    elif weakref_slot is not False:  # pragma: <3.11 cover
         raise TypeError("make_dataclass() got an unexpected keyword argument 'weakref_slot'")
 
-    if sys.version_info >= (3, 12):
+    if sys.version_info >= (3, 12):  # pragma: >=3.12 cover
         if module is None:
             try:
                 # pylint: disable-next=protected-access
@@ -446,15 +484,26 @@ def make_dataclass(  # type: ignore[no-redef] # noqa: C901
                     # pylint: disable-next=protected-access
                     module = sys._getframe(1).f_globals.get('__name__', '__main__')
         make_dataclass_kwargs['module'] = module
-    elif module is not None:
-        raise TypeError("make_dataclass() got an unexpected keyword argument'module'")
+    elif module is not None:  # pragma: <3.12 cover
+        raise TypeError("make_dataclass() got an unexpected keyword argument 'module'")
 
-    cls = dataclasses.make_dataclass(
+    registered_by_decorator = False
+    if sys.version_info >= (3, 14):  # pragma: >=3.14 cover
+        if decorator in (dataclasses.dataclass, dataclass):
+            decorator = functools.partial(dataclass, namespace=namespace)
+            registered_by_decorator = True
+        make_dataclass_kwargs['decorator'] = decorator
+    elif decorator is not dataclasses.dataclass:  # pragma: <3.14 cover
+        raise TypeError("make_dataclass() got an unexpected keyword argument 'decorator'")
+
+    cls: _TypeT = dataclasses.make_dataclass(  # type: ignore[assignment]
         cls_name,
         fields=fields,
         **dataclass_kwargs,  # type: ignore[arg-type]
         **make_dataclass_kwargs,  # type: ignore[arg-type]
     )
-    dataclass_kwargs.pop('slots', None)  # already defined in `make_dataclass()`
-    dataclass_kwargs.pop('weakref_slot', None)  # already used in `make_dataclass()`
-    return dataclass(cls, **dataclass_kwargs, namespace=namespace)  # type: ignore[call-overload]
+    if not registered_by_decorator:  # pragma: <3.14 cover
+        dataclass_kwargs.pop('slots', None)  # already defined in `make_dataclass()`
+        dataclass_kwargs.pop('weakref_slot', None)  # already used in `make_dataclass()`
+        cls = dataclass(cls, **dataclass_kwargs, namespace=namespace)  # type: ignore[call-overload]
+    return cls
