@@ -20,6 +20,7 @@ from __future__ import annotations
 
 import contextlib
 import dataclasses
+import functools
 import inspect
 import sys
 from collections import OrderedDict, defaultdict, deque, namedtuple
@@ -36,7 +37,15 @@ from optree.accessors import (
     SequenceEntry,
     StructSequenceEntry,
 )
-from optree.typing import PyTreeKind, StructSequence, T, is_namedtuple_class, is_structseq_class
+from optree.typing import (
+    Children,
+    MetaData,
+    PyTreeKind,
+    StructSequence,
+    T,
+    is_namedtuple_class,
+    is_structseq_class,
+)
 from optree.utils import safe_zip, total_order_sorted, unzip2
 
 
@@ -301,6 +310,11 @@ def register_pytree_node(
         ValueError: If the namespace is an empty string.
         ValueError: If the type is already registered in the registry.
 
+    .. versionadded:: 0.12.0
+        The ``path_entry_type`` argument to specify the path entry type used in
+        :meth:`PyTreeSpec.accessors` and :func:`tree_flatten_with_accessor`.
+        If not provided, :class:`AutoEntry` will be used.
+
     Examples:
         >>> # Registry a Python type with lambda functions
         >>> register_pytree_node(
@@ -434,6 +448,7 @@ def register_pytree_node_class(
 ) -> CustomTreeNodeType: ...
 
 
+# pylint: disable-next=too-many-branches
 def register_pytree_node_class(  # noqa: C901
     cls: CustomTreeNodeType | str | None = None,
     /,
@@ -470,9 +485,25 @@ def register_pytree_node_class(  # noqa: C901
     Raises:
         TypeError: If the path entry class is not a subclass of :class:`PyTreeEntry`.
         TypeError: If the namespace is not a string.
+        TypeError: If the class does not define the required method pairs.
         ValueError: If the namespace is an empty string.
         ValueError: If the type is already registered in the registry.
 
+    .. versionadded:: 0.12.0
+        The ``TREE_PATH_ENTRY_TYPE`` class variable to specify the path entry type used in
+        :meth:`PyTreeSpec.accessors` and :func:`tree_flatten_with_accessor`.
+        If not provided, :class:`AutoEntry` will be used.
+
+    .. versionadded:: 0.18.0
+        Previously, this function looked for methods named ``tree_flatten`` and ``tree_unflatten``
+        for the given class. Since version 0.18.0, it prefers methods named ``__tree_flatten__``
+        and ``__tree_unflatten__`` instead. The old method names are still supported for
+        backward compatibility, but it is recommended to use the new method names.
+        The method resolution follows this priority:
+        1. If both ``__tree_flatten__`` and ``__tree_unflatten__`` are defined, use them directly.
+        2. If both ``tree_flatten`` and ``tree_unflatten`` are defined, wrap them as dunder methods.
+        3. If neither complete pair is available, raise a :exc:`TypeError` suggesting the new method names.
+
     This function is a thin wrapper around :func:`register_pytree_node`, and provides a
     class-oriented interface:
 
@@ -486,22 +517,34 @@ def register_pytree_node_class(  # noqa: C901
                 self.x = x
                 self.y = y
 
-            def tree_flatten(self):
+            def __tree_flatten__(self):
                 return ((self.x, self.y), None, ('x', 'y'))
 
             @classmethod
-            def tree_unflatten(cls, metadata, children):
+            def __tree_unflatten__(cls, metadata, children):
                 return cls(*children)
 
         @register_pytree_node_class('mylist')
         class MyList(UserList):
             TREE_PATH_ENTRY_TYPE = SequenceEntry
 
+            def __tree_flatten__(self):
+                return self.data, None, None
+
+            @classmethod
+            def __tree_unflatten__(cls, metadata, children):
+                return cls(*children)
+
+        # Legacy style (still supported but not recommended)
+        @register_pytree_node_class(namespace='legacy')
+        class LegacyStyleMyList(UserList):
             def tree_flatten(self):
+                # Implementation automatically wrapped as __tree_flatten__
                 return self.data, None, None
 
             @classmethod
             def tree_unflatten(cls, metadata, children):
+                # Implementation automatically wrapped as __tree_unflatten__
                 return cls(*children)
     """
     if cls is __GLOBAL_NAMESPACE or isinstance(cls, str):
@@ -536,10 +579,49 @@ def register_pytree_node_class(  # noqa: C901
     if not (inspect.isclass(path_entry_type) and issubclass(path_entry_type, PyTreeEntry)):
         raise TypeError(f'Expected a subclass of PyTreeEntry, got {path_entry_type!r}.')
 
+    # Check for dunder-styled methods first (preferred since 0.18.0)
+    if not all(
+        callable(getattr(cls, method, None))
+        for method in ('__tree_flatten__', '__tree_unflatten__')
+    ):
+        # Check for old-styled methods (backward compatibility)
+        if not all(
+            callable(getattr(cls, method, None)) for method in ('tree_flatten', 'tree_unflatten')
+        ):
+            raise TypeError(
+                f'{cls!r} must define both `__tree_flatten__` and `__tree_unflatten__` methods '
+                'for registration as a pytree node.',
+            )
+
+        # Add dunder-styled wrapper methods to the class
+        # pylint: disable=no-member
+
+        @functools.wraps(cls.tree_flatten)
+        def __tree_flatten__(  # noqa: N807
+            self: CustomTreeNode[T],
+            /,
+        ) -> tuple[Children[T], MetaData] | tuple[Children[T], MetaData, Iterable[Any] | None]:
+            return self.tree_flatten()  # type: ignore[attr-defined]
+
+        @classmethod  # type: ignore[misc]
+        @functools.wraps(getattr(cls.tree_unflatten, '__func__', cls.tree_unflatten))
+        def __tree_unflatten__(  # noqa: N807
+            cls: type[CustomTreeNode[T]],
+            metadata: MetaData,
+            children: Children[T],
+            /,
+        ) -> CustomTreeNode[T]:
+            return cls.tree_unflatten(metadata, children)  # type: ignore[attr-defined]
+
+        # pylint: enable=no-member
+
+        cls.__tree_flatten__ = __tree_flatten__
+        cls.__tree_unflatten__ = __tree_unflatten__
+
     register_pytree_node(
         cls,
-        methodcaller('tree_flatten'),
-        cls.tree_unflatten,
+        methodcaller('__tree_flatten__'),
+        cls.__tree_unflatten__,
         path_entry_type=path_entry_type,
         namespace=namespace,
     )
@@ -725,8 +807,8 @@ def _defaultdict_flatten(
     tuple[Callable[[], VT] | None, list[KT]],
     tuple[KT, ...],
 ]:
-    values, keys, entries = _dict_flatten(dct)
-    return values, (dct.default_factory, keys), entries
+    values, dict_metadata, entries = _dict_flatten(dct)
+    return values, (dct.default_factory, dict_metadata), entries
 
 
 def _defaultdict_unflatten(
@@ -734,8 +816,8 @@ def _defaultdict_unflatten(
     values: Iterable[VT],
     /,
 ) -> defaultdict[KT, VT]:
-    default_factory, keys = metadata
-    return defaultdict(default_factory, _dict_unflatten(keys, values))
+    default_factory, dict_metadata = metadata
+    return defaultdict(default_factory, _dict_unflatten(dict_metadata, values))
 
 
 def _defaultdict_insertion_ordered_flatten(
@@ -746,8 +828,8 @@ def _defaultdict_insertion_ordered_flatten(
     tuple[Callable[[], VT] | None, list[KT]],
     tuple[KT, ...],
 ]:
-    values, keys, entries = _dict_insertion_ordered_flatten(dct)
-    return values, (dct.default_factory, keys), entries
+    values, dict_metadata, entries = _dict_insertion_ordered_flatten(dct)
+    return values, (dct.default_factory, dict_metadata), entries
 
 
 def _defaultdict_insertion_ordered_unflatten(
@@ -755,8 +837,8 @@ def _defaultdict_insertion_ordered_unflatten(
     values: Iterable[VT],
     /,
 ) -> defaultdict[KT, VT]:
-    default_factory, keys = metadata
-    return defaultdict(default_factory, _dict_insertion_ordered_unflatten(keys, values))
+    default_factory, dict_metadata = metadata
+    return defaultdict(default_factory, _dict_insertion_ordered_unflatten(dict_metadata, values))
 
 
 def _deque_flatten(deq: deque[T], /) -> tuple[deque[T], int | None]:
